#!/usr/local/bin/python
import sys
import time
import datetime
import sets

class logdict (dict):
    def count(self,key):
        return len(self[key])

    def add(self, key, value):
        try:
            item = self[key]
        except KeyError:
            item = list()
            self[key] = item
        item.append(value)

    def sum(self, key1, key2):
        sum = 0
        found = False
        for run in self[key1]:
            try:
                sum += run[key2]
                found = True
            except KeyError:
                pass
        if not found:
            raise KeyError
        return sum

    def avg(self, key1, key2):
        return self.sum(key1, key2)/self.count(key1)
    
    def max(self, key1, key2):
        runs = self[key1]
        max = None
        for run in runs:
            try:
                val = run[key2]
            except KeyError:
                pass
            else:
                if max is None or max < val:
                    max = val
        if max is None:
            raise KeyError
        return max

    def min(self, key1, key2):
        runs = self[key1]
        min = None
        for run in runs:
            try:
                val = run[key2]
            except KeyError:
                pass
            else:
                if min is None or min > val:
                    min = val
        if min is None:
            raise KeyError
        return min

    def unique(self, key1, key2):
        runs = self[key1]
        vals = []
        for run in runs:
            try:
                val = run[key2]
            except KeyError:
                pass
            else:
                if val not in vals:
                    vals.append(val)
        if not vals:
            raise KeyError
        return vals
    
def printmap(keytext, valmap, keyname='', mode=None, days=0):
    valkeys = valmap.keys()
    valkeys.sort()
    if keyname:
        if mode == 'hour':
            head = ('Daily','Daily','Daily')
        else:
            head = ('','Total','Total')
        
        print '                   %5s   %5s    %5s  Bad     Avg.   Long     Secs' % head
        if mode == 'user':
            userhdr = ''
        else:
            userhdr = 'Users'
        
        print ' %7s     %5s  Runs  Runtime    Jobs  Jobs  Runtime Runtime   /Job' % \
              (keyname, userhdr)
    for k in valkeys:
        secs = valmap.sum(k, 'runtime')
        jobs = valmap.sum(k, 'jobs')
        runs = valmap.count(k)
        if mode == 'hour':
            totmins,totsecs = divmod(secs / days, 60)
            totjobs = jobs / days
            runs = float(runs) / days
            runfmt = '%5.1f'
        else:
            totmins,totsecs = divmod(secs, 60)
            totjobs = jobs
            runfmt = '%5d'
        tothrs,totmins = divmod(totmins, 60)
        runmins,runsecs = divmod(valmap.avg(k, 'runtime'), 60)
        longmins, longsecs = divmod(valmap.max(k, 'runtime'), 60)
        if jobs == 0:
            perjob = 'n/a'
        else:
            perjob = '%7.3f' % (secs / jobs)
        if mode == 'user':
            users = '     '
        else:
            users = '%5d' % len(valmap.unique(k,'user'))
        print((keytext +
               '%5s '+ runfmt + ' %2d:%0.2d:%04.1f %7d %3d   %2d:%04.1f %2d:%04.1f %7s') %
              (k, users, runs, tothrs, totmins, totsecs,
               totjobs, valmap.sum(k, 'badjobs'), runmins, runsecs,
               longmins, longsecs, perjob))

def parse_log(logfile, logissues, start=None, end=None):
    users = logdict()
    allmap = logdict()
    daymap = logdict()
    hourmap = logdict()
    statusmap = logdict()
    for line in logfile:
        pieces = line.split()
        logdate = time.localtime()[0:1] + \
                  time.strptime(' '.join(pieces[0:3]),"%b %d %H:%M:%S")[1:]
        if (start and logdate < start) or (end and logdate > end):
            continue
        if pieces[4].strip(':') != 'nc':
            continue
        if pieces[5].strip(':') != '/usr/local/bin/nc':
            continue
        pieces = pieces[6:]
        info = dict()
        i = 0
        status = 0
        
        while i < len(pieces):
            parts = pieces[i].split('=')
            i += 1

            if parts[0] == 'errcode' or parts[0] == 'status':
                status = 1
                if not logissues:
                    break
                info[parts[0]] = parts[1]
            elif parts[0] == 'start':
                secs = pieces[i].split('.')
                starttime = time.strptime(parts[1] + ' ' + secs[0],
                                                  "%m/%d/%y %H:%M:%S")
                info['startdate'] = (starttime.tm_year, starttime.tm_mon,
                                     starttime.tm_mday)
                info['starttime'] = (starttime.tm_hour, starttime.tm_min,
                                     starttime.tm_sec)
                i += 1
            elif parts[0] == 'run':
                secs = parts[1].split('.')
                runtime = time.strptime(secs[0],
                                    "%H:%M:%S")
                info['runtime'] = (runtime.tm_hour * 60 + runtime.tm_min) * \
                                  60 + runtime.tm_sec + float('0.' + secs[1])
            elif parts[0] in ['jobs', 'bigjobs', 'badjobs']:
                info[parts[0]] = int(parts[1])
            elif parts[0] == 'user':
                user = parts[1]
                info['user'] = user
            else:
                try:
                    info[parts[0]] = parts[1]
                except IndexError:
                    info[parts[0]] = None
        if status:
            if not logissues:
                continue
            statusmap.add(logdate, info)
            status = 0
            continue

        users.add(user, info)

        #if info['jobs'] > info['badjobs'] or info['badjobs'] == 0:
        # only count runs with at least 1 good job, or
        # at least no bad jobs
        allmap.add('Total', info)
        daymap.add(datetime.date(starttime.tm_year, starttime.tm_mon,
                                 starttime.tm_mday), info)

        hourmap.add(starttime.tm_hour, info)
    printmap('%12s', users, keyname='User', mode='user')
    print
    printmap('%12s', daymap, keyname='Time')
    printmap('%12s', allmap)
    print
    printmap('     Hour %02d', hourmap, keyname='Time', mode='hour',
             days = len(daymap.items()))
    if logissues and statusmap:
        print
        print 'Issues:'
        statusmap = statusmap.items()
        statusmap.sort()
        for k,v in statusmap:
            print time.strftime('%m/%d/%y %H:%M:%S', k), v
            print
        
if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('','--issues', action='store_true',
                      help='show logged issues')
    options, args = parser.parse_args()

    try:
        inputfile = file(args[0])
    except IndexError:
        inputfile = sys.stdin
    parse_log(inputfile, logissues=options.issues)
    
